package ru.dbotthepony.mc.otm.client.screen.panels.button

import com.mojang.blaze3d.platform.InputConstants
import ru.dbotthepony.mc.otm.block.entity.MatteryDeviceBlockEntity
import ru.dbotthepony.mc.otm.block.entity.RedstoneSetting
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.client.render.Widgets18
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel
import ru.dbotthepony.mc.otm.client.screen.panels.FramePanel
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.math.RelativeSide
import ru.dbotthepony.mc.otm.menu.input.IPlayerInputWithFeedback
import ru.dbotthepony.mc.otm.menu.input.ItemHandlerPlayerInput
import java.util.function.Predicate

private fun <S : MatteryScreen<*>> makeRedstoneSettingButton(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	control: IPlayerInputWithFeedback<RedstoneSetting>
): LargeEnumRectangleButtonPanel<S, RedstoneSetting> {
	return object : LargeEnumRectangleButtonPanel<S, RedstoneSetting>(
		screen, parent, x = x, y = y,
		defaultValue = RedstoneSetting.LOW, enum = RedstoneSetting::class.java,
		prop = control,
	) {
		override var isDisabled: Boolean
			get() = !control.test(minecraft.player)
			set(value) {}

		init {
			add(RedstoneSetting.IGNORED, tooltip = RedstoneSetting.IGNORED.description, skinElement = Widgets18.REDSTONE_IGNORED)
			add(RedstoneSetting.LOW, tooltip = RedstoneSetting.LOW.description, skinElement = Widgets18.REDSTONE_LOW)
			add(RedstoneSetting.HIGH, tooltip = RedstoneSetting.HIGH.description, skinElement = Widgets18.REDSTONE_HIGH)
		}
	}
}

private fun <S : MatteryScreen<*>> makeItemModeButton(screen: S, parent: FramePanel<S>, input: ItemHandlerPlayerInput.Piece): LargeEnumRectangleButtonPanel<S, MatteryDeviceBlockEntity.ItemHandlerMode> {
	val button = LargeEnumRectangleButtonPanel(screen, parent, enum = MatteryDeviceBlockEntity.ItemHandlerMode::class.java, prop = input.input, defaultValue = input.default)

	val values = listOf(
		MatteryDeviceBlockEntity.ItemHandlerMode.DISABLED to Widgets18.DISABLED,
		MatteryDeviceBlockEntity.ItemHandlerMode.INPUT to Widgets18.INPUT_ONLY,
		MatteryDeviceBlockEntity.ItemHandlerMode.OUTPUT to Widgets18.OUTPUT_ONLY,
		MatteryDeviceBlockEntity.ItemHandlerMode.INPUT_OUTPUT to Widgets18.INPUT_OUTPUT,
		MatteryDeviceBlockEntity.ItemHandlerMode.BATTERY to Widgets18.BATTERY_ONLY,
	)

	for ((k, v) in values) {
		button.add(k, skinElement = v, tooltip = TranslatableComponent(k.translationKey))
	}

	button.finish()
	button.predicate = Predicate { input.isAllowed(it) }

	return button
}

private fun <S : MatteryScreen<*>> makeItemHandlerControlPanel(
	screen: S,
	inputs: ItemHandlerPlayerInput
): FramePanel<S> {
	val frame = object : FramePanel<S>(screen, 78f, 80f, TranslatableComponent("otm.gui.sides.item_config")) {
		override fun tickInner() {
			super.tickInner()

			if (!isEverFocused()) {
				remove()
			}
		}
	}

	val front = makeItemModeButton(screen, frame, inputs.pieces[RelativeSide.FRONT]!!)
	val back = makeItemModeButton(screen, frame, inputs.pieces[RelativeSide.BACK]!!)
	val left = makeItemModeButton(screen, frame, inputs.pieces[RelativeSide.LEFT]!!)
	val right = makeItemModeButton(screen, frame, inputs.pieces[RelativeSide.RIGHT]!!)
	val top = makeItemModeButton(screen, frame, inputs.pieces[RelativeSide.TOP]!!)
	val bottom = makeItemModeButton(screen, frame, inputs.pieces[RelativeSide.BOTTOM]!!)

	if (inputs.pull.test(minecraft.player)) {
		val pull = LargeBooleanRectangleButtonPanel(
			screen,
			frame,
			skinElementActive = Widgets18.PULL,
			skinElementInactive = Widgets18.PULL_DISABLED,
			prop = inputs.pull,
		)

		pull.tooltip = TranslatableComponent("otm.gui.side_mode.pull")

		pull.x = 30f - 20f
		pull.y = 14f
	}

	if (inputs.push.test(minecraft.player)) {
		val push = LargeBooleanRectangleButtonPanel(
			screen,
			frame,
			skinElementActive = Widgets18.PUSH,
			skinElementInactive = Widgets18.PUSH_DISABLED,
			prop = inputs.push,
		)

		push.tooltip = TranslatableComponent("otm.gui.side_mode.push")

		push.x = 30f + 20f
		push.y = 14f
	}

	top.tooltip = TranslatableComponent("otm.gui.sides.top")
	bottom.tooltip = TranslatableComponent("otm.gui.sides.bottom")
	back.tooltip = TranslatableComponent("otm.gui.sides.back")
	front.tooltip = TranslatableComponent("otm.gui.sides.front")
	left.tooltip = TranslatableComponent("otm.gui.sides.left")
	right.tooltip = TranslatableComponent("otm.gui.sides.right")

	top.x = 30f
	top.y = 14f

	right.x = 30f - 20f
	right.y = 14f + 20f

	left.x = 30f + 20f
	left.y = 14f + 20f

	front.x = 30f
	front.y = 14f + 20f

	bottom.x = 30f
	bottom.y = 14f + 42f

	back.x = 30f - 20f
	back.y = 14f + 42f

	screen.addPanel(frame)

	frame.requestFocus()

	return frame
}

fun <S : MatteryScreen<*>> makeDeviceControls(
	screen: S,
	parent: FramePanel<S>,
	redstone: IPlayerInputWithFeedback<RedstoneSetting>? = null,
	itemConfig: ItemHandlerPlayerInput? = null
): EditablePanel<S> {
	val panel = object : EditablePanel<S>(screen, parent, width = LargeEnumRectangleButtonPanel.SIZE, height = 0f, x = parent.width + 3f) {
		override fun tickInner() {
			super.tickInner()
			x = parent.width + 3f
			y = 0f
		}
	}

	var y = 0f

	if (redstone != null) {
		y += makeRedstoneSettingButton(screen, panel, y = y, control = redstone).height + 2f
	}

	if (itemConfig != null) {
		y += object : LargeRectangleButtonPanel<S>(screen, panel, y = y, skinElement = Widgets18.ITEMS_CONFIGURATION) {
			init {
				tooltip = TranslatableComponent("otm.gui.sides.item_config")
			}

			override fun onClick(mouseButton: Int) {
				if (mouseButton == InputConstants.MOUSE_BUTTON_LEFT) {
					val frame = makeItemHandlerControlPanel(screen, itemConfig)

					frame.x = absoluteX + width / 2f - frame.width / 2f
					frame.y = absoluteY + height + 8f
				}
			}
		}.height + 2f
	}

	panel.height = (y - 2f).coerceAtLeast(0f)
	return panel
}
